// This file was generated by ecm_setup_version(): DO NOT EDIT!

#ifndef SOLID_VERSION_H
#define SOLID_VERSION_H

#define SOLID_VERSION_STRING "6.24.0"
#define SOLID_VERSION_MAJOR 6
#define SOLID_VERSION_MINOR 24
#define SOLID_VERSION_PATCH 0
#define SOLID_VERSION ((6<<16)|(24<<8)|(0))

#endif
